/*********************************************
 *                                           *
 *         Programa Editor em Janela         *
 *         Modulo de manipulacao de arquivos *
 *                                           *
 *********************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <curses.h>

#include "edicao.h"

/* Estrutura que associa um arquivo a uma area de edicao */ 
struct winfile {
  char *nome;
  WINDOW *win;
};

static struct winfile *files = (struct winfile *) NULL;
static int numFiles = 0;

/* Retorna o comprimento de uma linha em uma janela especificada */
static int len (WINDOW *win, int line)
{
  int col;

  for (col = COLS / 2 - 2; col >=0 &&
	 mvwinch (win, line, col) == ESPACO; --col) ;

  return (col + 1);
}

/* Retorna o numero de linhas em uma janela */
static int scrsize (WINDOW *win)
{
  int line;

  for (line = LINES - 3; line >= 0 &&
	                 len (win, line) == 0; --line) ;

  return (line + 1);
}


/* Abre arquivo, copia ateh LINES - 2 linhas para a janela especificada */
static void getfile (struct winfile file)
{
  int line;
  char linebuf[512],
       mensagem[512] = "Nao posso ler arquivo ";
  FILE *infile;

  if ((infile = fopen (file.nome, "r")) == NULL) {
    /* Envia mensagem de erro no final da tela */
    printMsg(strcat(mensagem,file.nome));
    return;
  }

  /* Leitura de LINES - 1 linhas do arquivo */
  for ( line = 0; line < LINES - 1 &&
	          fgets (linebuf, COLS / 2, infile) != NULL; ++line) {
    /* coloca linha lida na tela */
    mvwaddstr (file.win, line, 0, linebuf);
  }

  fclose (infile);
  wmove (file.win, 0, 0);
  wrefresh(file.win);
}


/* Abre arquivo, copia conteudo da janela para arquivo */
static void putfile (struct winfile file)
{
  int line,
      col,
      linelen,
      pagelen;
  char mensagem[512] = "Nao posso gravar o arquivo ";
  FILE *outfile;

  if ((outfile = fopen (file.nome, "w")) == NULL) {
    /* Envia mensagem de erro no final da tela */
    printMsg(strcat(mensagem,file.nome));
    return;
  }

  pagelen = scrsize (file.win);

  /* Leitura de LINES - 2 linhas do arquivo */
  for ( line = 0;  line < pagelen; ++line) {
    linelen =  len(file.win, line);
    for ( col = 0; col < linelen; ++col) {
      putc (mvwinch (file.win, line, col), outfile);
    }

    putc (LF, outfile);
  }

  fclose (outfile);
}

/* Associa areas de edicao a um conjunto de arquivos.
 * A area de edicao deve ter sido previamente criada com setWindows()
 * (edicao.c)
 * ARGUMENTOS:  nomes:   array de strings com os nomes dos arquivos
 *              numfile: quantidade de strings em "nomes"
 * VARIAVEIS GLOBAIS:  file
 */
static void setFiles(int numfile, char *nomes[])
{
  int i;

  if (! files)
    files = (struct winfile *) calloc (0,sizeof(struct winfile));
  
  for (i=0; i < numfile; ++i) {
    files = (struct winfile *) realloc (files, (i + 1) * sizeof(struct winfile));
    files[i].nome = strdup(nomes[i]);
    files[i].win = getWindow(i);
  }
}


/* Carrega nas duas areas de edicao os arquivos indicados 
 * ARGUMENTOS:  nomes:   array de strings com os nomes dos arquivos
 *              numfile: quantidade de strings em "nomes"
 */
void loadFiles(int numfile, char *nomes[])
{
  int i;

/* Associa uma area de edicao a um nome de arquivo */
  setFiles(numfile, nomes);

  for (i=0; i < numfile; i++) {
    getfile(files[i]);
  }

  numFiles = numfile;
}


/* Salva todos os arquivos em edicao */
void saveFiles()
{
  int i;

  for (i=0; i < numFiles; i++) {
    putfile(files[i]);
  }

}

